﻿using System;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing09
{
  /// <summary>
  /// Sample of a main driver program
  /// </summary>
  public class Program
  {
    public Program()
    {
      var salesRegSvc = new SaleRegistrationService();
      salesRegSvc.CompleteSale("username", Guid.NewGuid(), 4);
    }
  }
  /// <summary>
  /// Service that registers a sale of a product
  /// </summary>
  public class SaleRegistrationService
  {
    /// <summary>
    /// Method called when a salesperson makes a sale
    /// </summary>
    public string CompleteSale(string salespersonUserName, Guid productId,
      int quantity)
    {
      //sale of some products is restricted
      //Ensure this salesperson is allows to sell this product
      var security = ObjectFactory.CreateSecurityService();
      if (!security.CanUserSellProduct(salespersonUserName, productId))
        return "Error - No permission to sell this product";

      //make sure there is enough product on hand
      var repo = ObjectFactory.CreateRepository();
      var canRemoveFromInventory = repo.RemoveFromInventory(productId, quantity);
      if (!canRemoveFromInventory)
        return "Error - not enough of that product in inventory";

      repo.FinalizeSale(salespersonUserName, productId, quantity);
      return "success";
    }
  }

  public interface ISecurityService
  {
    bool CanUserSellProduct(string username, Guid productId);
  }

  /// <summary>
  /// Class that provides all access to security
  /// </summary>
  public class SecurityService : ISecurityService
  {
    private string _federatedSecurityUrl;
    public SecurityService(string federatedSecurityUrl)
    {
      _federatedSecurityUrl = federatedSecurityUrl;
    }
    /// <summary>
    /// Decide if the given username has access
    /// to the requested object
    /// </summary>
    public bool CanUserSellProduct(string username, Guid productId)
    {
      //super user can do anything
      if (username == "SuperSalesPerson") return true;

      var permissionsString = GetUserPermissionsFromURL(_federatedSecurityUrl);
      if (permissionsString.Contains("Inactive")) return false;

      var repo = ObjectFactory.CreateRepository();
      return (repo.DoesUserHavePermission(username, productId));
    }

    /// <summary>
    /// Retrieve a permissions string from the external registration site
    /// </summary>
    private string GetUserPermissionsFromURL(string url)
    {
      //use federated security URL to get permission string
      return "permissions go here";
    }
  }


  public interface IRepository
  {
    bool RemoveFromInventory(Guid productId, int quantity);
    void FinalizeSale(string salesPersonName, Guid productId, int quantity);
    bool IsUserActive(string username);
    bool DoesUserHavePermission(string username, Guid productId);
  }

  /// <summary>
  /// Repository class that controls all interaction
  /// with the database
  /// </summary>
  public class Repository : IRepository
  {
    public bool RemoveFromInventory(Guid productId, int quantity)
    {
      //query the DB to see if there is enough inventory 
      //for this transaction
      return true;
    }

    public void FinalizeSale(string salesPersonName, Guid productId, int quantity)
    {
      //finalize the sale by saving info to the database
      //for who sold the product and how many of them
    }

    public bool IsUserActive(string username)
    {
      //query the DB to see if this user is active
      return true;
    }

    public bool DoesUserHavePermission(string username, Guid productId)
    {
      //query the DB to see if this user has 
      //sufficient permissions
      return true;
    }
  }

  /// <summary>
  /// Abstract factory class to create services
  /// </summary>
  public static class ObjectFactory
  {
    /// <summary>
    /// This property will be set at application startup
    /// or user login when it is known what Url to use
    /// </summary>
    public static string SecurityUrl { get; set; }

    // *** This method can now return a test service if in debug mode
    public static ISecurityService CreateSecurityService()
    {
#if Debug
      return new TestSecurityService(SecurityUrl);
#else
      return new SecurityService(SecurityUrl);
#endif
    }

    public static IRepository CreateRepository()
    {
      return new Repository();
    }
  }

  public class TestSecurityService : ISecurityService
  {
    public TestSecurityService(string pFilename)
    {
      //don't need to save anything for the test service
    }

    public bool CanUserSellProduct(string username, Guid productId)
    {
      //Always return falst for test purposes
      return true;
    }
  }

}