﻿using System;
using Microsoft.Practices.Unity;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing15
{
  /// <summary>
  /// Sample of a main driver program
  /// </summary>
  public class Program
  {
    public Program()
    {
      var salesRegSvc = ServiceLocator.Resolve<ISaleRegistrationService>();
      salesRegSvc.CompleteSale("username", Guid.NewGuid(), 4);
    }
  }

  public interface ISaleRegistrationService
  {
    /// <summary>
    /// Method called when a salesperson makes a sale
    /// </summary>
    string CompleteSale(string salespersonUserName, Guid productId,
                                        int quantity);
  }

  /// <summary>
  /// Service that registers a sale of a product
  /// </summary>
  public class SaleRegistrationService : ISaleRegistrationService
  {
    /// <summary>
    /// Method called when a salesperson makes a sale
    /// </summary>
    public string CompleteSale(string salespersonUserName, Guid productId,
      int quantity)
    {
      //sale of some products is restricted
      //Ensure this salesperson is allows to sell this product
      var security = ServiceLocator.Resolve<ISecurityService>();
      if (!security.CanUserSellProduct(salespersonUserName, productId))
        return "Error - No permission to sell this product";

      //make sure there is enough product on hand
      var repo = ServiceLocator.Resolve<IRepository>();
      var canRemoveFromInventory = repo.RemoveFromInventory(productId, quantity);
      if (!canRemoveFromInventory)
        return "Error - not enough of that product in inventory";

      repo.FinalizeSale(salespersonUserName, productId, quantity);
      return "success";
    }
  }

  public interface ISecurityService
  {
    bool CanUserSellProduct(string username, Guid productId);
  }

  /// <summary>
  /// Class that provides all access to security
  /// </summary>
  public class SecurityService : ISecurityService
  {
    private string _federatedSecurityUrl;
    public SecurityService(string federatedSecurityUrl)
    {
      _federatedSecurityUrl = federatedSecurityUrl;
    }
    /// <summary>
    /// Decide if the given username has access
    /// to the requested object
    /// </summary>
    public bool CanUserSellProduct(string username, Guid productId)
    {
      //super user can do anything
      if (username == "SuperSalesPerson") return true;

      var permissionsString = GetUserPermissionsFromURL(_federatedSecurityUrl);
      if (permissionsString.Contains("Inactive")) return false;

      var repo = ServiceLocator.Resolve<IRepository>();
      return (repo.DoesUserHavePermission(username, productId));
    }

    /// <summary>
    /// Retrieve a permissions string from the external registration site
    /// </summary>
    private string GetUserPermissionsFromURL(string url)
    {
      //use federated security URL to get permission string
      return "permissions go here";
    }
  }


  public interface IRepository
  {
    bool RemoveFromInventory(Guid productId, int quantity);
    void FinalizeSale(string salesPersonName, Guid productId, int quantity);
    bool IsUserActive(string username);
    bool DoesUserHavePermission(string username, Guid productId);
  }

  /// <summary>
  /// Repository class that controls all interaction
  /// with the database
  /// </summary>
  public class Repository : IRepository
  {
    public bool RemoveFromInventory(Guid productId, int quantity)
    {
      //query the DB to see if there is enough inventory 
      //for this transaction
      return true;
    }

    public void FinalizeSale(string salesPersonName, Guid productId, int quantity)
    {
      //finalize the sale by saving info to the database
      //for who sold the product and how many of them
    }

    public bool IsUserActive(string username)
    {
      //query the DB to see if this user is active
      return true;
    }

    public bool DoesUserHavePermission(string username, Guid productId)
    {
      //query the DB to see if this user has 
      //sufficient permissions
      return true;
    }
  }

  public static class ServiceLocator
  {
    private static UnityContainer _container;

    static ServiceLocator()
    {
      _container = new UnityContainer();

      //by adding the parameter in the RegisterType call,
      //we have made the Repository a singleton
      _container.RegisterType<IRepository, Repository>(
        new ContainerControlledLifetimeManager());

      //if a class needs setup logic, it can be created and the
      //instanced registered with the container
      var securitySvc = new SecurityService("security URL goes here");
      _container.RegisterInstance<ISecurityService>(securitySvc);
    }

    public static TProxyType Resolve<TProxyType>()
    {
      return _container.Resolve<TProxyType>();
    }
  }


}