﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using Microsoft.Practices.Prism.Logging;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing07
{
  /// <summary>
  /// Logger implementation. 
  /// This class should go in the Services project
  /// </summary>
  public class Logger : ILogger
  {
    private string _logFilePath;

    /// <summary>
    /// Constructor - initializes the location of the log file to the
    /// same folder where the application is running.
    /// </summary>
    public Logger()
    {
      _logFilePath = Path.GetDirectoryName(Application.ExecutablePath)
        + "\\MyLogFile.log";
    }

    /// <summary>
    /// Log a message to the log file
    /// </summary>
    public void Log(string format, params object[] arg)
    {
      Log(String.Format(format, arg));
    }

    /// <summary>
    /// Log a message to the log file
    /// </summary>
    public void Log(string pMessage)
    {
      writeLogInfo(pMessage, Category.Info, Priority.None);
    }

    /// <summary>
    /// Log an exception to the log file
    /// </summary>
    public void Log(Exception pException)
    {
      if (pException == null) return;
      writeLogInfo(pException.ToString(), Category.Exception, Priority.High);
    }

    /// <summary>
    /// Log a message to the log file
    /// </summary>
    public void Log(string pMessage, Category pCategory, Priority pPriority)
    {
      writeLogInfo(pMessage, pCategory, pPriority);
    }

    /// <summary>
    /// Writes the log message to the log file
    /// </summary>
    internal void writeLogInfo(string pMessage, Category pCategory,
      Priority pPriority)
    {
      if (string.IsNullOrEmpty(pMessage))
        pMessage = "Empty message received in LogWriterService";

      //for 10,000 test calls, adding the stack frame 
      //added 0.5 seconds to exe time so getting stack 
      //frame each time adds negligible performance hit.
      var callingStackFrame = new StackFrame(1);  //1 for our caller 

      string msg = String.Format("{0} - {1} - {2}",
        pMessage, pCategory, pPriority);
      using (var file = new StreamWriter(_logFilePath))
      {
        file.WriteLine(callingStackFrame + Environment.NewLine + msg);
        file.Close();
      }
    }
  }

  /// <summary>
  /// This interface should go in the Interfaces project
  /// </summary>
  public interface ILogger
  {
    void Log(string msg);
    void Log(Exception exc);
    void Log(string format, params object[] arg);
    void Log(string pMessage, Category pCategory, Priority pPriority);
  }
}