﻿using System;
using System.Drawing;
using System.Reflection;
using System.Resources;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing11
{
  public class ResourceProvider : IResourceProvider
  {
    internal Assembly _currAssembly;

    /// <summary>
    /// Value to return if the requested string is not found
    /// </summary>
    public string MissingStringValue { get; set; }

    /// <summary>
    /// Value to return if the requested Icon is not found
    /// </summary>
    public Icon MissingIconValue { get; set; }

    /// <summary>
    /// Value to return if the requested Image is not found
    /// </summary>
    public Image MissingImageValue { get; set; }

    internal ResourceManager ResourceMgr
    {
      get
      {
        return _resourceMgr ??
          (_resourceMgr = new ResourceManager(
          "Resources.Resource1", _currAssembly));
      }
    }
    internal ResourceManager _resourceMgr;

    public ResourceProvider()
    {
      _currAssembly = Assembly.GetExecutingAssembly();
    }

    /// <summary>
    /// Returns a resource identified by the given name.  If the resource
    /// is not found, it will make an attempt to return something useful.
    /// </summary>
    public T GetResource<T>(string pName) where T : class
    {
      var obj = ResourceMgr.GetObject(pName) as T;
      if (obj != null) return obj;

      return HandleObjectNotFoundError<T>(pName);
    }

    private T HandleObjectNotFoundError<T>(string pName) where T : class
    {
      if (typeof(T) == typeof(string))
      {
        if (MissingStringValue == null)
          throw new ArgumentOutOfRangeException(
            "String resource not found " + pName);
        return MissingStringValue as T;
      }

      if (typeof(T) == typeof(Icon))
      {
        if (MissingIconValue == null)
          throw new ArgumentOutOfRangeException(
            "Icon resource not found " + pName);
        return MissingIconValue as T;
      }

      if (typeof(T) == typeof(Image))
      {
        if (MissingImageValue == null)
          throw new ArgumentOutOfRangeException(
            "Image resource not found " + pName);
        return MissingImageValue as T;
      }

      throw new ArgumentException("Unsupported object type " + typeof(T));
    }
  }


  public interface IResourceProvider
  {
    T GetResource<T>(string pName) where T : class;
    string MissingStringValue { get; set; }
    Icon MissingIconValue { get; set; }
    Image MissingImageValue { get; set; }
  }

  public class ResourceProviderExample
  {
    public ResourceProviderExample()
    {
      var svc = ServiceLocator.Resolve<IResourceProvider>();
      svc.MissingStringValue = "missing string value";
    }

    public string GetStringResourceExample()
    {
      var svc = ServiceLocator.Resolve<IResourceProvider>();
      return svc.GetResource<string>("String resource name");
    }
  }
}
