﻿using System;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using Microsoft.Practices.Prism.Events;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing12
{

  public interface IMessageAggregator : IEventAggregator
  {
  }

  /// <summary>
  /// Provides messaging services for the application.
  /// </summary>
  public class MessageAggregator : EventAggregator, IMessageAggregator
  {
    // This is an extension point for the Prism EventAggregator.  It allows
    // us to enhance the EventAggregator later if necessary without having
    // to change any calling code.
  }

  /// <summary>
  /// Dto class that carries the payload for a status message
  /// </summary>
  public class StatusMessageDto
  {
    public StatusMessageDto(string pMsg)
    {
      Message = pMsg;
    }

    //put any kind of relevant information here that is necessary
    public string Message { get; set; }
  }


  /// <summary>
  /// Container class that organizes all the messages for the system
  /// </summary>
  public class MessageEvents
  {
    /// <summary>
    /// This message is used when the caller wants to set app status
    /// </summary>
    public class StatusMessage : CompositePresentationEvent<StatusMessageDto>
    {
    }
  }

  public class MessageAggregatorExample
  {
    /// <summary>
    /// This publishes a message to all subscribers.
    /// </summary>
    public void PublishMessage()
    {
      var msgAgg = ServiceLocator.Resolve<IMessageAggregator>();
      var msg = msgAgg.GetEvent<MessageEvents.StatusMessage>();
      var dto = new StatusMessageDto("Hello World");
      msg.Publish(dto);
    }

    /// <summary>
    /// This listens for publications having the given message type.
    /// </summary>
    public void SimpleSubscription()
    {
      var msgAgg = ServiceLocator.Resolve<IMessageAggregator>();
      var msg = msgAgg.GetEvent<MessageEvents.StatusMessage>();
      var subscriptionToken = msg.Subscribe(SetApplicationStatus);
    }

    /// <summary>
    /// This listens for publications having the given message type
    /// and filters to only those starting with the word "Hello"
    /// </summary>
    public void FilteredSubscription()
    {
      var msgAgg = ServiceLocator.Resolve<IMessageAggregator>();
      var msg = msgAgg.GetEvent<MessageEvents.StatusMessage>();
      msg.Subscribe(

        //method to call with payload
        SetApplicationStatus,

        //thread to use for the subscription
        ThreadOption.PublisherThread,

        //strong reference flag
        false,

        //filter for the message
        pParam => pParam.Message.StartsWith("Hello"));
    }

    /// <summary>
    /// Process the message as appropriate
    /// </summary>
    public void SetApplicationStatus(StatusMessageDto pStatusMessage)
    {
      Console.WriteLine(pStatusMessage.Message);
    }

  }
}
