﻿using System;
using System.Collections.Generic;
using System.Linq;
using CodeSamples.Ch12_RefactorToAdvancedServices.Listing10;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing11
{
  public interface IObjectManager<T> where T : class
  {
    T Register(T newObject, string objectName);
    T Get(string pObjectName, bool pIncludeInactive = true);
    T Register(IObjectManagerReference<T> pObjectReference);
    bool Remove(string pObjectName);
  }

  /// <summary>
  /// Manages all the objects registered with it
  /// while optionally allowing them to be garbage 
  /// collected when appropriate.
  /// </summary>
  /// <remarks>
  /// There is a small mem leak here in that the Dto 
  /// used to keep track of the weak reference items 
  /// is not cleaned up when the item itself is.
  /// This could be extended to add a Cleanup method that will release
  /// unused Dto's but that is left as an exercise for the reader.
  /// </remarks>
  public class ObjectManager<T> : IObjectManager<T> where T : class
  {
    /// <summary>
    /// Indicator whether caller wants to use 
    /// strong or weak references
    /// </summary>
    internal bool _useStrongReference;

    /// <summary>
    /// Items being managed
    /// </summary>
    internal Dictionary<string, IObjectManagerReference<T>> _items;

    /// <summary>
    /// Constructor
    /// </summary>
    public ObjectManager(bool pStrongReference = true)
    {
      _useStrongReference = pStrongReference;
      _items = new Dictionary<string, IObjectManagerReference<T>>();
    }

    /// <summary>
    /// Registers a new item with the manager
    /// and returns the registered item (to enable 
    /// fluent interfaces)
    /// </summary>
    public T Register(T pNewObject, string pObjectName)
    {
      if (_useStrongReference)
        return Register(new StrongReferenceDto<T>(pNewObject, pObjectName));
      return Register(new WeakReferenceDto<T>(pNewObject, pObjectName));
    }

    /// <summary>
    /// Registers a new item with the manager
    /// and returns the registered item (to enable 
    /// fluent interfaces)
    /// </summary>
    public T Register(IObjectManagerReference<T> pObjectReference)
    {
      if (pObjectReference.Object == null)
        throw new ArgumentNullException("pNewObject");
      if (string.IsNullOrEmpty(pObjectReference.Name))
        throw new ArgumentNullException("pObjectName");

      if (_items.ContainsKey(pObjectReference.Name))
        throw new ArgumentException(pObjectReference.Name
          + " key already exists.");

      _items.Add(pObjectReference.Name, pObjectReference);
      return pObjectReference.Object;
    }

    /// <summary>
    /// Returns the named item to the caller
    /// </summary>
    public T Get(string pObjectName, bool pIncludeInactive = true)
    {
      if (pIncludeInactive)
        return (from x in _items
                where (x.Key.Equals(pObjectName))
                select x.Value.Object).FirstOrDefault();

      return (from x in _items
              where (x.Value.IsAlive && x.Key.Equals(pObjectName))
              select x.Value.Object).FirstOrDefault();
    }

    /// <summary>
    /// Removes the first occurrence of the object from the manager
    /// </summary>
    /// <returns>
    /// True if item is successfully removed; otherwise, false. 
    /// </returns>
    public bool Remove(string pObjectName)
    {
      //must be a valid name
      if (string.IsNullOrEmpty(pObjectName))
        throw new ArgumentNullException("pObjectName");

      //if we can't find the name, object is not in the list
      if (!_items.ContainsKey(pObjectName)) return true;

      return _items.Remove(pObjectName);
    }
  }
}