﻿using System;
using System.Collections.Generic;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using CodeSamples.Ch12_RefactorToAdvancedServices.Listing12;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing14
{
  /// <summary>
  /// This class will take a command line, parse it into
  /// the component parts, then process each command.
  /// </summary>
  public class CommandLineInterpreter
  {
    /// <summary>
    /// CommandDispatcher we will use for executing the commands
    /// </summary>
    internal ICommandDispatcherService<string> _dispatcher;

    /// <summary>
    /// Constructor - this will populate the list of commands.
    /// </summary>
    public CommandLineInterpreter()
    {
      _dispatcher = ServiceLocator
        .Resolve<ICommandDispatcherService<string>>();
      _dispatcher.KeyNotFoundCommand = new MissingKeyCommand();
      _dispatcher.AddCommand(new Cmd1Command());
      //add more commands here
    }

    /// <summary>
    /// This method executes a single command
    /// </summary>
    /// <param name="pCommandLine">The command to be executed</param>
    public void Execute(string pCommandLine)
    {
      if (string.IsNullOrEmpty(pCommandLine)) return;
      pCommandLine = pCommandLine.Trim();
      var commandQueue = ParseCommandLineIntoCommands(pCommandLine);
      while (commandQueue.Count > 0)
      {
        var cmd = commandQueue.Dequeue();
        _dispatcher.ExecuteCommand(cmd, commandQueue);
      }
    }

    /// <summary>
    /// Parses into separate commands by splitting on spaces.  If support
    /// for quoted strings and embedded spaces is necessary, 
    /// this logic must be added.
    /// </summary>
    internal Queue<string> ParseCommandLineIntoCommands(string pCommandLine)
    {
      var commands = pCommandLine.Split(' ');
      var result = new Queue<string>();
      foreach (var cmd in commands)
        result.Enqueue(cmd);
      return result;
    }

    /// <summary>
    /// Command to do something important
    /// </summary>
    internal class Cmd1Command : BaseDispatchedCommand<string>
    {
      //give the base the string used to invoke this command
      public Cmd1Command() : base("/cmd1") { }
      public override void Execute(object parameter)
      {
        //do something important
      }
    }


    /// <summary>
    /// Command that is executed if the incoming command from the 
    /// command line is not found.  This command can do nothing,
    /// log an error, or anything else that is appropriate.
    /// </summary>
    internal class MissingKeyCommand : BaseDispatchedCommand<string>
    {
      public MissingKeyCommand() : base(Guid.NewGuid().ToString()) { }
      public override void Execute(object parameter)
      {
        //do error handling here
      }
    }
  }

  public interface ICommandDispatcherService<T>
  {
    void ExecuteCommand(T pCommandLine, object pParameter);
    void AddCommand(IDispatchedCommand<T> pCommand);
    IDispatchedCommand<T> KeyNotFoundCommand { get; set; }
    void ClearCommands();
  }

}
