﻿using System;
using System.Windows.Forms;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using CodeSamples.Ch11_RefactorToBasicServices.Listing01;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing06
{
  public class BusinessLogicClass
  {
    private IDialogService DialogSvc
    {
      get
      {
        return _dialogSvc ?? (
          _dialogSvc = ServiceLocator.Resolve<IDialogService>());
      }
    }
    private IDialogService _dialogSvc;


    /// <summary>
    /// Example of a method that can be refactored to use
    /// the dialog service.
    /// </summary>
    public bool ValidateDateRange(DateTime pStartDate, DateTime pEndDate)
    {
      if (pEndDate < pStartDate)
      {
        DialogSvc.Show("End Date must be after Start Date");
        return false;
      }
      return true;
    }
  }
  
  [TestClass]
  public class MockingExampleTestClass
  {
    private BusinessLogicClass _bizLogic;
    private Mock<IDialogService> _mockDialogSvc;

    [ClassInitialize]
    public static void ClassInitialize(TestContext context)
    {
      var bootStrapper = new BootStrapper();
      bootStrapper.Run();
    }

    [TestInitialize]
    public void TestInitialize()
    {
      //Create a mock DialogService 
      //that will do nothing when called
      _mockDialogSvc = new Mock<IDialogService>();

      //Replace the real DialogService with our mock.
      ServiceLocator.Container = new UnityContainer();
      ServiceLocator.RegisterInstance<IDialogService>(
        _mockDialogSvc.Object);

      _bizLogic = new BusinessLogicClass();
    }

    [TestMethod]
    public void ValidateDateRange_ValidDates_NoMessage()
    {
      var startDt = new DateTime(2000, 1, 1);
      var endDt = new DateTime(2000, 1, 2);
      _bizLogic.ValidateDateRange(startDt, endDt);

      //ensure that the message was NOT shown to the user
      _mockDialogSvc.Verify(mds => 
        mds.Show(It.IsAny<string>()), Times.Never());
    }

    [TestMethod]
    public void ValidateDateRange_SameDates_NoMessage()
    {
      var startDt = new DateTime(2000, 1, 2);
      _bizLogic.ValidateDateRange(startDt, startDt);

      //ensure that the message was NOT shown to the user
      _mockDialogSvc.Verify(mds => 
        mds.Show(It.IsAny<string>()), Times.Never());
    }

    [TestMethod]
    public void ValidateDateRange_InvalidDates_ShowsMessage()
    {
      var startDt = new DateTime(2000, 1, 2);
      var endDt = new DateTime(2000, 1, 1);
      _bizLogic.ValidateDateRange(startDt, endDt);

      //ensure that the message was NOT shown to the user
      _mockDialogSvc.Verify(mds => 
        mds.Show(It.IsAny<string>()), Times.Once());
    }
  }

  /// <summary>
  /// The reader should remove this BootStrapper and replace with the normal one 
  /// used to setup the application.
  /// </summary>
  public class BootStrapper
  {
    public void Run(){}
  }

}